/*
 * DHT12 connect to port A
 * SPS30 RX connect to port 16, TX connect to port 17 or port C if your core is m5stack fire
 */



#include <M5Stack.h>
#include "DHT12.h"
#include <Wire.h> //The DHT12 uses I2C comunication.
#include "sensirion_uart.h"
#include "sps30.h"

DHT12 dht12; //Preset scale CELSIUS and ID 0x5c.
byte dmode=2;  //default display mode 0=info 1=temp&hum 2=PM
float temp=0;
float hum=0;
float pm1=0;
float pm25=0;
float pm4=0;
float pm10=0;
float avp=0; //averate particle size

void createmenu() {
  M5.Lcd.setCursor(30, 225);
  M5.Lcd.setTextColor(WHITE, BLACK);
  M5.Lcd.setTextSize(2);
  M5.Lcd.printf(".Info.");
  M5.Lcd.setCursor(127, 225);
  M5.Lcd.printf(".Temp.");
  M5.Lcd.setCursor(235, 225);
  M5.Lcd.printf(".PM.");
}

void info() {
  M5.Lcd.setCursor(0, 5);
  M5.Lcd.setTextColor(BLACK, WHITE);
  M5.Lcd.setTextSize(3);
  M5.Lcd.printf(" Temp: %2.1f C.\r\n", temp);
  M5.Lcd.printf(" Humi: %2.0f percent", hum);
  M5.Lcd.setCursor(0, 70);
  M5.Lcd.setTextColor(BLACK, WHITE);
  M5.Lcd.setTextSize(3);
  M5.Lcd.printf(" PM 1.0 : %2.2f \r\n", pm1);
  M5.Lcd.printf(" PM 2.5 : %2.2f \r\n", pm25);
  M5.Lcd.printf(" PM 4.0 : %2.2f \r\n", pm4);
  M5.Lcd.printf(" PM 10  : %2.2f \r\n", pm10);
  M5.Lcd.setTextColor(BLACK,WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.printf("\r\n  AV particle size:%2.2f", avp);
}

void pmGraph() {
  int pm25Level;
  int pm10Level;
  M5.Lcd.setCursor(0, 15);
  M5.Lcd.setTextColor(BLACK, WHITE);
  M5.Lcd.setTextSize(3);
  M5.Lcd.printf(" PM  2.5 : %2.0f    ", pm25);
  M5.Lcd.setCursor(0, 120);
  M5.Lcd.printf(" PM 10.0 : %2.0f    ", pm10);
  pm25Level=pm25;
  pm10Level=pm10;
  if (pm25Level>320) pm25Level=320;
  if (pm10Level>320) pm10Level=320;
  //draw graph pm2.5 bg
  M5.Lcd.fillRect(0, 50, 320, 10, WHITE);  //clear bg
  M5.Lcd.fillRect(0, 100, 320, 10, WHITE);  //clear bg
  M5.Lcd.fillRect(0, 60, 25, 40, CYAN);
  M5.Lcd.fillRect(25, 60, 12, 40, GREEN);
  M5.Lcd.fillRect(37, 60, 13, 40, YELLOW);
  M5.Lcd.fillRect(50, 60, 40, 40, ORANGE);
  M5.Lcd.fillRect(90, 60, 230, 40, RED);
  M5.Lcd.fillRect(pm25Level-2, 50, 4, 60, BLACK);     //draw level
  //draw graph pm10 bg
  M5.Lcd.fillRect(0, 150, 320, 10, WHITE);  //clear bg
  M5.Lcd.fillRect(0, 200, 320, 10, WHITE);  //clear bg
  M5.Lcd.fillRect(0, 160, 50, 40, CYAN);
  M5.Lcd.fillRect(50, 160, 30, 40, GREEN);
  M5.Lcd.fillRect(80, 160, 40, 40, YELLOW);
  M5.Lcd.fillRect(120, 160, 60, 40, ORANGE);
  M5.Lcd.fillRect(180, 160, 140, 40, RED);
  M5.Lcd.fillRect(pm10Level-2, 150, 4, 60, BLACK);     //draw level
}

void setup()
{
  M5.begin();
  Wire.begin();
  M5.Lcd.fillScreen(WHITE);
  createmenu(); 
  Serial.begin(115200);
  sensirion_uart_open();
  while (sps30_probe() != 0) {
  M5.Lcd.setCursor(5, 5);
  M5.Lcd.setTextColor(RED, WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.printf("probe failed");
  Serial.println("probe failed");
  delay(500);
  }
  sps30_set_fan_auto_cleaning_interval(15*60);  //auto clean every 15 min.
  /* start measurement and wait for 10s to ensure the sensor has a
  * stable flow and possible remaining particles are cleaned out */
  if (sps30_start_measurement() != 0) {
      //M5.Lcd.setCursor(0, 0);
      //M5.Lcd.setTextColor(RED, WHITE);
      //M5.Lcd.setTextSize(2);
      //M5.Lcd.printf("error starting");
      Serial.println("error starting measurement");
  }
}

void loop()
{
  M5.update();
  struct sps30_measurement measurement;
  s16 ret;
  ret = sps30_read_measurement(&measurement);
  temp = dht12.readTemperature();
  hum = dht12.readHumidity();
  if (ret < 0) {
    Serial.println("read measurement failed");
  } else {
    if (SPS_IS_ERR_STATE(ret)) {
      Serial.println("Measurements may not be accurate");
    }else{
      pm1  = measurement.mc_1p0;
      pm25 = measurement.mc_2p5;
      pm4  = measurement.mc_4p0;
      pm10 = measurement.mc_10p0;
      avp  = measurement.typical_particle_size;
      Serial.print("pm1.0 :");Serial.println(pm1);
      Serial.print("pm2.5 :");Serial.println(pm25);
      Serial.print("pm4.0 :");Serial.println(pm4);
      Serial.print("pm10.0 :");Serial.println(pm10);
      Serial.print("nc0.5 :");Serial.println(measurement.nc_0p5);
      Serial.print("nc1.0 :");Serial.println(measurement.nc_1p0);
      Serial.print("nc2.5 :");Serial.println(measurement.nc_2p5);
      Serial.print("nc4.5 :");Serial.println(measurement.nc_4p0);
      Serial.print("nc10.0 :");Serial.println(measurement.nc_10p0);
      Serial.print("typical particle size :");Serial.println(avp);
    }
  }
  
  //buttom event
  if (M5.BtnA.wasReleased()) {
    Serial.println("button A");
    dmode=0;
    M5.Lcd.fillScreen(WHITE);
    createmenu();
  } else if (M5.BtnB.wasReleased()) {
    Serial.println("button B");
    dmode=1;
    M5.Lcd.fillScreen(WHITE);
    drawScale();
    createmenu();
  } else if (M5.BtnC.wasReleased()) {
    Serial.println("button C");
    dmode=2;
    M5.Lcd.fillScreen(WHITE);
    createmenu();
  }
  
  //mode select
  if (dmode==0) info();
  if (dmode==1) tempGraph();
  if (dmode==2) pmGraph();
}

void tempGraph() {
  M5.Lcd.setCursor(0, 15);
  M5.Lcd.setTextColor(BLACK, WHITE);
  M5.Lcd.setTextSize(3);
  M5.Lcd.printf(" T = %2.0f | H = %2.0f    ", temp, hum);
  drawScale();
  M5.Lcd.fillRect(55, 75, 200, 100, WHITE);   //clear gb grid for re-draw pointer
  drawTempHumZone();
  drawSubGrid();
  TempHumPlot();
}

void drawScale() {
  // text temp scale
  M5.Lcd.setTextColor(BLACK, WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(30, 50);
  M5.Lcd.printf("T");
  M5.Lcd.setCursor(20, 70);
  M5.Lcd.printf("35");
  M5.Lcd.setCursor(20, 95);
  M5.Lcd.printf("30");
  M5.Lcd.setCursor(20, 120);
  M5.Lcd.printf("25");
  M5.Lcd.setCursor(20, 145);
  M5.Lcd.printf("20");
  M5.Lcd.setCursor(20, 170);
  M5.Lcd.printf("15");
  // text hum scale
  M5.Lcd.setTextColor(BLACK, WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(60, 200);
  M5.Lcd.printf("20");
  M5.Lcd.setCursor(90, 200);
  M5.Lcd.printf("30");
  M5.Lcd.setCursor(120, 200);
  M5.Lcd.printf("40");
  M5.Lcd.setCursor(150, 200);
  M5.Lcd.printf("50");
  M5.Lcd.setCursor(180, 200);
  M5.Lcd.printf("60");
  M5.Lcd.setCursor(210, 200);
  M5.Lcd.printf("70");
  M5.Lcd.setCursor(240, 200);
  M5.Lcd.printf("80");
  M5.Lcd.setCursor(280, 200);
  M5.Lcd.printf("H");
  //draw axis
  M5.Lcd.fillRect(50, 70, 4, 120, BLUE);  //draw Y axis
  M5.Lcd.fillRect(50, 190, 210, 4, BLUE);  //draw X axis
}

void drawSubGrid() {
  //draw grid X
  M5.Lcd.fillRect(50, 78, 210, 1, BLACK);
  M5.Lcd.fillRect(50, 103, 210, 1, BLACK);
  M5.Lcd.fillRect(50, 128, 210, 1, BLACK);
  M5.Lcd.fillRect(50, 153, 210, 1, BLACK);
  M5.Lcd.fillRect(50, 178, 210, 1, BLACK);
  //draw grid Y
  M5.Lcd.fillRect(70, 70, 1, 120, BLACK);
  M5.Lcd.fillRect(100, 70, 1, 120, BLACK);
  M5.Lcd.fillRect(130, 70, 1, 120, BLACK);
  M5.Lcd.fillRect(160, 70, 1, 120, BLACK);
  M5.Lcd.fillRect(190, 70, 1, 120, BLACK);
  M5.Lcd.fillRect(220, 70, 1, 120, BLACK);
  M5.Lcd.fillRect(250, 70, 1, 120, BLACK);  
}

void drawTempHumZone() {
  for (int i=0; i<=40; i++) {
    M5.Lcd.drawLine(100,103+i,190,128+i,CYAN);
  }
}

void TempHumPlot() {
  int originX=50;
  int originY=190;
  int x;  //X = hum
  int y;  //Y = temp
  float limitTemp;
  float limitHum;
  limitTemp = temp;
  limitHum = hum;
  if (limitTemp<15) limitTemp=15;
  if (limitTemp>35) limitTemp=35;
  if (limitHum<20) limitTemp=20;
  if (limitHum>80) limitTemp=80;
  x = int(map(limitHum,20,80,70,250));
  //y = int(map(limitTemp,15,35,78,178));
  y = int(map(limitTemp,15,35,178,78));
  Serial.print("X = ");Serial.println(x);
  Serial.print("Y = ");Serial.println(y);
  M5.Lcd.fillRect(x-4, y-4, 7, 7, RED);  
}

